@@ -1,5 +1,5 @@
 /* source: filan_main.c */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* Copyright Gerhard Rieger 2001-2008 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 const char copyright[] = "filan by Gerhard Rieger - see http://www.dest-unreach.org/socat/";
@@ -23,7 +23,8 @@ static void filan_usage(FILE *fd);
 int main(int argc, const char *argv[]) {
    const char **arg1, *a;
    const char *filename = NULL, *waittimetxt;
-   unsigned int m = 0, n = 1024;	/* this is default on my Linux */
+   unsigned int m = 0;		/* first FD (default) */
+   unsigned int n = FD_SETSIZE;	/* last excl. */
    unsigned int i;
    int style = 0;
    struct timespec waittime = { 0, 0 };
@@ -57,7 +58,7 @@ int main(int argc, const char *argv[]) {
 	    }
 	 }
          m = strtoul(a, (char **)&a, 0);
-	 n = m+1;
+	 n = m;
 	 break;
       case 'n': if (arg1[0][2]) {
 	    a = *arg1+2;
@@ -127,11 +128,11 @@ int main(int argc, const char *argv[]) {
       exit(1);
    }
    if (outfname) {
-      // special cases
+      /* special cases */
       if (!strcmp(outfname,"stdin")) { fdout=stdin; }
       else if (!strcmp(outfname,"stdout")) { fdout=stdout; }
       else if (!strcmp(outfname,"stderr")) { fdout=stderr; }
-      // file descriptor
+      /* file descriptor */
       else if (*outfname == '+') {
 	 a = outfname+1;
 	 fildes = strtoul(a, (char **)&a, 0);
@@ -140,7 +141,7 @@ int main(int argc, const char *argv[]) {
 	    exit(1);
 	 }
       } else {
-	 // file name
+	 /* file name */
 	 if ((fdout = fopen(outfname, "w")) == NULL) {
 	    Error2("can't fopen '%s': %s\n",
 		   outfname, strerror(errno));
@@ -168,6 +169,9 @@ int main(int argc, const char *argv[]) {
 #endif
 	 filan_file(filename, fdout);
       } else {
+	 if (m == n) {
+	    ++n;
+	 }
 	 for (i = m; i < n; ++i) {
 	    filan_fd(i, fdout);
 	 }
@@ -189,12 +193,16 @@ int main(int argc, const char *argv[]) {
 	    Debug2("open(\"%s\", O_RDONLY|O_NOCTTY|O_NONBLOCK|O_LARGEFILE, 0700): %s",
 		   filename, strerror(errno));
 	 }
-	 fdname(filename, fd, fdout);
+	 fdname(filename, fd, fdout, NULL);
 #endif
-	 fdname(filename, -1, fdout);
+	 fdname(filename, -1, fdout, NULL);
       } else {
-	 for (i = m; i < n; ++i) {
-	    fdname("", i, fdout);
+	 if (m == n) {
+	    fdname("", m, fdout, NULL);
+	 } else {
+	    for (i = m; i < n; ++i) {
+	       fdname("", i, fdout, "%5u ");
+	    }
 	 }
       }
    }
@@ -225,7 +233,7 @@ static void filan_usage(FILE *fd) {
    fputs("      -ls            log to stderr (default if no other log)\n", fd);
 #endif
    fputs("      -i<fdnum>      only analyze this fd\n", fd);
-   fputs("      -n<fdnum>      analyze all fds from 0 up to fdnum-1 (default: 1024)\n", fd);
+   fprintf(fd, "      -n<fdnum>      analyze all fds from 0 up to fdnum-1 (default: %u)\n", FD_SETSIZE);
    fputs("      -s             simple output with just type and socket address or path\n", fd);
 /*   fputs("      -c             alternate device visualization\n", fd);*/
    fputs("      -f<filename>   analyze file system entry\n", fd);
