@@ -1,5 +1,5 @@
-/* $Id: socat.c,v 1.111 2007/03/06 21:03:28 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2007 */
+/* source: socat.c */
+/* Copyright Gerhard Rieger 2001-2009 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this is the main source, including command line option parsing, general
@@ -83,12 +83,16 @@ bool havelock;
 
 int main(int argc, const char *argv[]) {
    const char **arg1, *a;
+   char *mainwaitstring;
    char buff[10];
    double rto;
    int i, argc0, result;
    struct utsname ubuf;
    int lockrc;
 
+   if (mainwaitstring = getenv("SOCAT_MAIN_WAIT")) {
+       sleep(atoi(mainwaitstring));
+   }
    diag_set('p', strchr(argv[0], '/') ? strrchr(argv[0], '/')+1 : argv[0]);
 
    /* we must init before applying options because env settings have lower
@@ -263,12 +267,14 @@ int main(int argc, const char *argv[]) {
       Error("-U and -u must not be combined");
    }
 
+   xioinitialize2();
    Info(copyright_socat);
 #if WITH_OPENSSL
    Info(copyright_openssl);
    Info(copyright_ssleay);
 #endif
    Debug2("socat version %s on %s", socatversion, timestamp);
+   xiosetenv("VERSION", socatversion, 1);	/* SOCAT_VERSION */
    uname(&ubuf);	/* ! here we circumvent internal tracing (Uname) */
    Debug4("running on %s version %s, release %s, machine %s\n",
 	   ubuf.sysname, ubuf.version, ubuf.release, ubuf.machine);
@@ -422,6 +428,16 @@ void socat_version(FILE *fd) {
 #else
    fputs("  #undef WITH_RAWIP\n", fd);
 #endif
+#ifdef WITH_GENERICSOCKET
+   fprintf(fd, "  #define WITH_GENERICSOCKET %d\n", WITH_GENERICSOCKET);
+#else
+   fputs("  #undef WITH_GENERICSOCKET\n", fd);
+#endif
+#ifdef WITH_INTERFACE
+   fprintf(fd, "  #define WITH_INTERFACE %d\n", WITH_INTERFACE);
+#else
+   fputs("  #undef WITH_INTERFACE\n", fd);
+#endif
 #ifdef WITH_TCP
    fprintf(fd, "  #define WITH_TCP %d\n", WITH_TCP);
 #else
@@ -432,6 +448,11 @@ void socat_version(FILE *fd) {
 #else
    fputs("  #undef WITH_UDP\n", fd);
 #endif
+#ifdef WITH_SCTP
+   fprintf(fd, "  #define WITH_SCTP %d\n", WITH_SCTP);
+#else
+   fputs("  #undef WITH_SCTP\n", fd);
+#endif
 #ifdef WITH_LISTEN
    fprintf(fd, "  #define WITH_LISTEN %d\n", WITH_LISTEN);
 #else
@@ -642,37 +663,33 @@ int socat(const char *address1, const char *address2) {
    returns >0 if child died and left data
 */
 int childleftdata(xiofile_t *xfd) {
-   fd_set in, out, expt;
+   struct pollfd in;
    int retval;
+
    /* have to check if a child process died before, but left read data */
    if (XIO_READABLE(xfd) &&
        (XIO_RDSTREAM(xfd)->howtoend == END_KILL ||
 	XIO_RDSTREAM(xfd)->howtoend == END_CLOSE_KILL ||
 	XIO_RDSTREAM(xfd)->howtoend == END_SHUTDOWN_KILL) &&
        XIO_RDSTREAM(xfd)->para.exec.pid == 0) {
-      struct timeval time0 = { 0,0 };
+      struct timeval timeout = { 0, 0 };
 
-      FD_ZERO(&in); FD_ZERO(&out); FD_ZERO(&expt);
       if (XIO_READABLE(xfd) && !(XIO_RDSTREAM(xfd)->eof >= 2 && !XIO_RDSTREAM(xfd)->ignoreeof)) {
-	 FD_SET(XIO_GETRDFD(xfd), &in);
-	 /*0 FD_SET(XIO_GETRDFD(xfd), &expt);*/
+	 in.fd = XIO_GETRDFD(xfd);
+	 in.events = POLLIN/*|POLLRDBAND*/;
+	 in.revents = 0;
       }
       do {
-	 retval = Select(FOPEN_MAX, &in, &out, &expt, &time0);
+	 retval = xiopoll(&in, 1, &timeout);
       } while (retval < 0 && errno == EINTR);
 
       if (retval < 0) {
-#if HAVE_FDS_BITS
-	 Error5("select(%d, &0x%lx, &0x%lx, &0x%lx, {0}): %s",
-		FOPEN_MAX, in.fds_bits[0], out.fds_bits[0],
-		expt.fds_bits[0], strerror(errno));
-#else
-	 Error5("select(%d, &0x%lx, &0x%lx, &0x%lx, {0}): %s",
-		FOPEN_MAX, in.__fds_bits[0], out.__fds_bits[0],
-		expt.__fds_bits[0], strerror(errno));
-#endif
+	 Error5("xiopoll({%d,%0o}, 1, {"F_tv_sec"."F_tv_usec"}): %s",
+		in.fd, in.events, timeout.tv_sec, timeout.tv_usec,
+		strerror(errno));
 	 return -1;
-      } else if (retval == 0) {
+      }
+      if (retval == 0) {
 	 Info("terminated child did not leave data for us");
 	 XIO_RDSTREAM(xfd)->eof = 2;
 	 xfd->stream.eof = 2;
@@ -685,21 +702,25 @@ int childleftdata(xiofile_t *xfd) {
 int xiotransfer(xiofile_t *inpipe, xiofile_t *outpipe,
 		unsigned char **buff, size_t bufsiz, bool righttoleft);
 
-bool mayrd1;		/* sock1 has read data or eof, according to select() */
-bool mayrd2;		/* sock2 has read data or eof, according to select() */
-bool maywr1;		/* sock1 can be written to, according to select() */
-bool maywr2;		/* sock2 can be written to, according to select() */
+bool mayrd1;		/* sock1 has read data or eof, according to poll() */
+bool mayrd2;		/* sock2 has read data or eof, according to poll() */
+bool maywr1;		/* sock1 can be written to, according to poll() */
+bool maywr2;		/* sock2 can be written to, according to poll() */
 
 /* here we come when the sockets are opened (in the meaning of C language),
    and their options are set/applied
    returns -1 on error or 0 on success */
 int _socat(void) {
-   fd_set in, out, expt;
+   struct pollfd fds[4],
+       *fd1in  = &fds[0],
+       *fd1out = &fds[1],
+       *fd2in  = &fds[2],
+       *fd2out = &fds[3];
    int retval;
    unsigned char *buff;
    ssize_t bytes1, bytes2;
    int polling = 0;	/* handling ignoreeof */
-   int wasaction = 1;	/* last select was active, do NOT sleep before next */
+   int wasaction = 1;	/* last poll was active, do NOT sleep before next */
    struct timeval total_timeout;	/* the actual total timeout timer */
 
 #if WITH_FILAN
@@ -757,10 +778,6 @@ int _socat(void) {
       /* for ignoreeof */
       if (polling) {
 	 if (!wasaction) {
-	    /* yes we could do it with select but I like readable trace output */
-	    if (socat_opts.pollintv.tv_sec)  Sleep(socat_opts.pollintv.tv_sec);
-	    if (socat_opts.pollintv.tv_usec) Usleep(socat_opts.pollintv.tv_usec);
-
 	    if (socat_opts.total_timeout.tv_sec != 0 ||
 		socat_opts.total_timeout.tv_usec != 0) {
 	       if (total_timeout.tv_usec < socat_opts.pollintv.tv_usec) {
@@ -796,14 +813,14 @@ int _socat(void) {
 
       if (closing>=1) {
 	 /* first eof already occurred, start end timer */
-	 timeout = socat_opts.closwait;
+	 timeout = socat_opts.pollintv;
 	 to = &timeout;
 	 closing = 2;
       }
 
-      do {
+      /* frame 1: set the poll parameters and loop over poll() EINTR) */
+      do {	/* loop over poll() EINTR */
 	 int _errno;
-	 FD_ZERO(&in); FD_ZERO(&out); FD_ZERO(&expt);
 
 	 childleftdata(sock1);
 	 childleftdata(sock2);
@@ -815,33 +832,64 @@ int _socat(void) {
 	    closing = 2;
 	 }
 
+	 /* use the ignoreeof timeout if appropriate */
+	 if (polling) {
+	    if (closing == 0 ||
+		(socat_opts.pollintv.tv_sec < timeout.tv_sec) ||
+		((socat_opts.pollintv.tv_sec == timeout.tv_sec) &&
+		 socat_opts.pollintv.tv_usec < timeout.tv_usec)) {
+	       timeout = socat_opts.pollintv;
+	    }
+	 }
+
+	 /* now the fds will be assigned */
 	 if (XIO_READABLE(sock1) &&
 	     !(XIO_RDSTREAM(sock1)->eof > 1 && !XIO_RDSTREAM(sock1)->ignoreeof) &&
 	     !socat_opts.righttoleft) {
-	    if (!mayrd1) {
-	       FD_SET(XIO_GETRDFD(sock1), &in);
+	    if (!mayrd1 && !(XIO_RDSTREAM(sock1)->eof > 1)) {
+		fd1in->fd = XIO_GETRDFD(sock1);
+		fd1in->events = POLLIN;
+	    } else {
+		fd1in->fd = -1;
 	    }
 	    if (!maywr2) {
-	       FD_SET(XIO_GETWRFD(sock2), &out);
+		fd2out->fd = XIO_GETWRFD(sock2);
+		fd2out->events = POLLOUT;
+	    } else {
+		fd2out->fd = -1;
 	    }
+	 } else {
+	     fd1in->fd = -1;
+	     fd2out->fd = -1;
 	 }
 	 if (XIO_READABLE(sock2) &&
 	     !(XIO_RDSTREAM(sock2)->eof > 1 && !XIO_RDSTREAM(sock2)->ignoreeof) &&
 	     !socat_opts.lefttoright) {
-	    if (!mayrd2) {
-	       FD_SET(XIO_GETRDFD(sock2), &in);
+	    if (!mayrd2 && !(XIO_RDSTREAM(sock2)->eof > 1)) {
+		fd2in->fd = XIO_GETRDFD(sock2);
+		fd2in->events = POLLIN;
+	    } else {
+		fd2in->fd = -1;
 	    }
 	    if (!maywr1) {
-	       FD_SET(XIO_GETWRFD(sock1), &out);
+		fd1out->fd = XIO_GETWRFD(sock1);
+		fd1out->events = POLLOUT;
+	    } else {
+		fd1out->fd = -1;
 	    }
+	 } else {
+	     fd1out->fd = -1;
+	     fd2in->fd = -1;
+	 }
+	 /* frame 0: innermost part of the transfer loop: check FD status */
+	 retval = xiopoll(fds, 4, to);
+	 if (retval >= 0 || errno != EINTR) {
+	    break;
 	 }
-	 retval = Select(FOPEN_MAX, &in, &out, &expt, to);
 	 _errno = errno;
-	 if (retval < 0 && errno == EINTR) {
-	    Info1("select(): %s", strerror(errno));
-	 }
+	 Info1("poll(): %s", strerror(errno));
 	 errno = _errno;
-      } while (retval < 0 && errno == EINTR);
+      } while (true);
 
       /* attention:
 	 when an exec'd process sends data and terminates, it is unpredictable
@@ -849,25 +897,24 @@ int _socat(void) {
 	 */
 
       if (retval < 0) {
-#if HAVE_FDS_BITS
-	    Error7("select(%d, &0x%lx, &0x%lx, &0x%lx, %s%lu): %s",
-		   FOPEN_MAX, in.fds_bits[0], out.fds_bits[0],
-		   expt.fds_bits[0], to?"&":"NULL/", to?to->tv_sec:0,
-		   strerror(errno));
-#else
-	    Error7("select(%d, &0x%lx, &0x%lx, &0x%lx, %s%lu): %s",
-		   FOPEN_MAX, in.__fds_bits[0], out.__fds_bits[0],
-		   expt.__fds_bits[0], to?"&":"NULL/", to?to->tv_sec:0,
-		   strerror(errno));
-#endif
+	 Error11("xiopoll({%d,%0o}{%d,%0o}{%d,%0o}{%d,%0o}, 4, {"F_tv_sec"."F_tv_usec"}): %s",
+		 fds[0].fd, fds[0].events, fds[1].fd, fds[1].events,
+		 fds[2].fd, fds[2].events, fds[3].fd, fds[3].events,
+		 timeout.tv_sec, timeout.tv_usec, strerror(errno));
 	    return -1;
       } else if (retval == 0) {
-	 Info2("select timed out (no data within %ld.%06ld seconds)",
+	 Info2("poll timed out (no data within %ld.%06ld seconds)",
 	       closing>=1?socat_opts.closwait.tv_sec:socat_opts.total_timeout.tv_sec,
 	       closing>=1?socat_opts.closwait.tv_usec:socat_opts.total_timeout.tv_usec);
 	 if (polling && !wasaction) {
 	    /* there was a ignoreeof poll timeout, use it */
-	    ;
+	    polling = 0;	/*%%%*/
+	    if (XIO_RDSTREAM(sock1)->ignoreeof) {
+	       mayrd1 = 0;
+	    }
+	 } else if (polling && wasaction) {
+	    wasaction = 0;
+
 	 } else if (socat_opts.total_timeout.tv_sec != 0 ||
 		    socat_opts.total_timeout.tv_usec != 0) {
 	    /* there was a total inactivity timeout */
@@ -884,17 +931,36 @@ int _socat(void) {
       }
 
       if (XIO_READABLE(sock1) && XIO_GETRDFD(sock1) >= 0 &&
-	  FD_ISSET(XIO_GETRDFD(sock1), &in)) {
+	  (fd1in->revents /*&(POLLIN|POLLHUP|POLLERR)*/)) {
+	 if (fd1in->revents & POLLNVAL) {
+	    /* this is what we find on Mac OS X when poll()'ing on a device or
+	       named pipe. a read() might imm. return with 0 bytes, resulting
+	       in a loop? */ 
+	    Error1("poll(...[%d]: invalid request", fd1in->fd);
+	    return -1;
+	 }
 	 mayrd1 = true;
       }
       if (XIO_READABLE(sock2) && XIO_GETRDFD(sock2) >= 0 &&
-	  FD_ISSET(XIO_GETRDFD(sock2), &in)) {
+	  (fd2in->revents)) {
+	 if (fd2in->revents & POLLNVAL) {
+	    Error1("poll(...[%d]: invalid request", fd2in->fd);
+	    return -1;
+	 }
 	 mayrd2 = true;
       }
-      if (XIO_GETWRFD(sock1) >= 0 && FD_ISSET(XIO_GETWRFD(sock1), &out)) {
+      if (XIO_GETWRFD(sock1) >= 0 && fd1out->fd >= 0 && fd1out->revents) {
+	 if (fd1out->revents & POLLNVAL) {
+	    Error1("poll(...[%d]: invalid request", fd1out->fd);
+	    return -1;
+	 }
 	 maywr1 = true;
       }
-      if (XIO_GETWRFD(sock2) >= 0 && FD_ISSET(XIO_GETWRFD(sock2), &out)) {
+      if (XIO_GETWRFD(sock2) >= 0 && fd2out->fd >= 0 && fd2out->revents) {
+	 if (fd2out->revents & POLLNVAL) {
+	    Error1("poll(...[%d]: invalid request", fd2out->fd);
+	    return -1;
+	 }
 	 maywr2 = true;
       }
 
@@ -913,13 +979,17 @@ int _socat(void) {
 	    maywr2 = false;
 	    total_timeout = socat_opts.total_timeout;
 	    wasaction = 1;
-	    /* is more data available that has already passed select()? */
+	    /* is more data available that has already passed poll()? */
 	    mayrd1 = (xiopending(sock1) > 0);
 	    if (XIO_RDSTREAM(sock1)->readbytes != 0 &&
 		XIO_RDSTREAM(sock1)->actbytes == 0) {
 	       /* avoid idle when all readbytes already there */
 	       mayrd1 = true;
-	    }          
+	    }
+	    /* escape char occurred? */
+	    if (XIO_RDSTREAM(sock1)->actescape) {
+	       bytes1 = 0;	/* indicate EOF */
+	    }
 	 }
 	 /* (bytes1 == 0)  handled later */
       } else {
@@ -941,13 +1011,17 @@ int _socat(void) {
 	    maywr1 = false;
 	    total_timeout = socat_opts.total_timeout;
 	    wasaction = 1;
-	    /* is more data available that has already passed select()? */
+	    /* is more data available that has already passed poll()? */
 	    mayrd2 = (xiopending(sock2) > 0);
 	    if (XIO_RDSTREAM(sock2)->readbytes != 0 &&
 		XIO_RDSTREAM(sock2)->actbytes == 0) {
 	       /* avoid idle when all readbytes already there */
 	       mayrd2 = true;
 	    }          
+	    /* escape char occurred? */
+	    if (XIO_RDSTREAM(sock2)->actescape) {
+	       bytes2 = 0;	/* indicate EOF */
+	    }
 	 }
 	 /* (bytes2 == 0)  handled later */
       } else {
@@ -956,32 +1030,53 @@ int _socat(void) {
 
       /* NOW handle EOFs */
 
+      /*0 Debug4("bytes1=F_Zd, XIO_RDSTREAM(sock1)->eof=%d, XIO_RDSTREAM(sock1)->ignoreeof=%d, closing=%d",
+	     bytes1, XIO_RDSTREAM(sock1)->eof, XIO_RDSTREAM(sock1)->ignoreeof,
+	     closing);*/
       if (bytes1 == 0 || XIO_RDSTREAM(sock1)->eof >= 2) {
-	 if (XIO_RDSTREAM(sock1)->ignoreeof && !closing) {
+	 if (XIO_RDSTREAM(sock1)->ignoreeof &&
+	     !XIO_RDSTREAM(sock1)->actescape && !closing) {
 	    Debug1("socket 1 (fd %d) is at EOF, ignoring",
 		   XIO_RDSTREAM(sock1)->fd);	/*! */
-	    polling = 1;
+	    mayrd1 = true;
+	    polling = 1;	/* do not hook this eof fd to poll for pollintv*/
 	 } else {
 	    Notice1("socket 1 (fd %d) is at EOF", XIO_GETRDFD(sock1));
 	    xioshutdown(sock2, SHUT_WR);
-	    if (socat_opts.lefttoright) {
-	       break;
-	    }
+	    XIO_RDSTREAM(sock1)->eof = 2;
+	    XIO_RDSTREAM(sock1)->ignoreeof = false;
 	 }
+      } else if (polling && XIO_RDSTREAM(sock1)->ignoreeof) {
+	 polling = 0;
+      }
+      if (XIO_RDSTREAM(sock1)->eof >= 2) {
+	 if (socat_opts.lefttoright) {
+	    break;
+	 }
+	 closing = 1;
       }
 
       if (bytes2 == 0 || XIO_RDSTREAM(sock2)->eof >= 2) {
-	 if (XIO_RDSTREAM(sock2)->ignoreeof && !closing) {
+	 if (XIO_RDSTREAM(sock2)->ignoreeof &&
+	     !XIO_RDSTREAM(sock2)->actescape && !closing) {
 	    Debug1("socket 2 (fd %d) is at EOF, ignoring",
 		   XIO_RDSTREAM(sock2)->fd);
-	    polling = 1;
+	    mayrd2 = true;
+	    polling = 1;	/* do not hook this eof fd to poll for pollintv*/
 	 } else {
 	    Notice1("socket 2 (fd %d) is at EOF", XIO_GETRDFD(sock2));
 	    xioshutdown(sock1, SHUT_WR);
-	    if (socat_opts.righttoleft) {
-	       break;
-	    }
+	    XIO_RDSTREAM(sock2)->eof = 2;
+	    XIO_RDSTREAM(sock2)->ignoreeof = false;
 	 }
+      } else if (polling && XIO_RDSTREAM(sock2)->ignoreeof) {
+	 polling = 0;
+      }
+      if (XIO_RDSTREAM(sock2)->eof >= 2) {
+	 if (socat_opts.righttoleft) {
+	    break;
+	 }
+	 closing = 1;
       }
    }
 
@@ -1065,8 +1160,9 @@ static int
 
 /* inpipe is suspected to have read data available; read at most bufsiz bytes
    and transfer them to outpipe. Perform required data conversions.
-   buff should be at least twice as large as bufsiz, to allow all standard
-   conversions. Returns the number of bytes written, or 0 on EOF or <0 if an
+   buff must be a malloc()'ed storage and might be realloc()'ed in this
+   function if more space is required after conversions. 
+   Returns the number of bytes written, or 0 on EOF or <0 if an
    error occurred or when data was read but none written due to conversions
    (with EAGAIN). EAGAIN also occurs when reading from a nonblocking FD where
    the file has a mandatory lock.
@@ -1076,9 +1172,9 @@ static int
 /* inpipe, outpipe must be single descriptors (not dual!) */
 int xiotransfer(xiofile_t *inpipe, xiofile_t *outpipe,
 		unsigned char **buff, size_t bufsiz, bool righttoleft) {
-   ssize_t bytes, writt;
+   ssize_t bytes, writt = 0;
 
-	 bytes = xioread(inpipe, *buff, socat_opts.bufsiz);
+	 bytes = xioread(inpipe, *buff, bufsiz);
 	 if (bytes < 0) {
 	    if (errno != EAGAIN)
 	       XIO_RDSTREAM(inpipe)->eof = 2;
@@ -1086,14 +1182,35 @@ int xiotransfer(xiofile_t *inpipe, xiofile_t *outpipe,
 	    return -1;
 	 }
 	 if (bytes == 0 && XIO_RDSTREAM(inpipe)->ignoreeof && !closing) {
-	    writt = 0;
+	    ;
 	 } else if (bytes == 0) {
 	    XIO_RDSTREAM(inpipe)->eof = 2;
 	    closing = MAX(closing, 1);
-	    writt = 0;
 	 }
 
-	 else /* if (bytes > 0)*/ {
+	 if (bytes > 0) {
+	    /* handle escape char */
+	    if (XIO_RDSTREAM(inpipe)->escape != -1) {
+	       /* check input data for escape char */
+	       unsigned char *ptr = *buff;
+	       size_t ctr = 0;
+	       while (ctr < bytes) {
+		  if (*ptr == XIO_RDSTREAM(inpipe)->escape) {
+		     /* found: set flag, truncate input data */
+		     XIO_RDSTREAM(inpipe)->actescape = true;
+		     bytes = ctr;
+		     Info("escape char found in input");
+		     break;
+		  }
+		  ++ptr; ++ctr;
+	       }
+	       if (ctr != bytes) {
+		  XIO_RDSTREAM(inpipe)->eof = 2;
+	       }
+	    }
+	 }
+
+	    if (bytes > 0) {
 
 	    if (XIO_RDSTREAM(inpipe)->lineterm !=
 		XIO_WRSTREAM(outpipe)->lineterm) {
@@ -1211,8 +1328,14 @@ int xiotransfer(xiofile_t *inpipe, xiofile_t *outpipe,
 #define LF '\n'
 
 
-int cv_newline(unsigned char **buff, ssize_t *bytes,
+/* converts the newline characters (or character sequences) from the one
+   specified in lineterm1 to that of lineterm2. Possible values are
+   LINETERM_CR, LINETERM_CRNL, LINETERM_RAW.
+   buff points to the malloc()'ed data, input and output. It may be subject to
+   realloc(). bytes specifies the number of bytes input and output */
+int cv_newline(unsigned char **buff, ssize_t *bufsiz,
 	       int lineterm1, int lineterm2) {
+   ssize_t *bytes = bufsiz;
    /* must perform newline changes */
    if (lineterm1 <= LINETERM_CR && lineterm2 <= LINETERM_CR) {
       /* no change in data length */
@@ -1250,7 +1373,7 @@ int cv_newline(unsigned char **buff, ssize_t *bytes,
 	    *t++ = *s++;
 	 }
       }
-      *bytes = t - *buff;
+      *bufsiz = t - *buff;
    } else {
       /* buffer becomes longer, must alloc another space */
       unsigned char *buf2;
@@ -1260,7 +1383,7 @@ int cv_newline(unsigned char **buff, ssize_t *bytes,
       } else {
 	 from = '\r';
       }
-      if ((buf2 = Malloc(2*socat_opts.bufsiz+1)) == NULL) {
+      if ((buf2 = Malloc(2*socat_opts.bufsiz/*sic!*/+1)) == NULL) {
 	 return -1;
       }
       s = *buff;  t = buf2;  z = *buff + *bytes;
@@ -1275,7 +1398,7 @@ int cv_newline(unsigned char **buff, ssize_t *bytes,
       }
       free(*buff);
       *buff = buf2;
-      *bytes = t - buf2;;
+      *bufsiz = t - buf2;;
    }
    return 0;
 }
