@@ -1,5 +1,5 @@
 /* source: sysutils.c */
-/* Copyright Gerhard Rieger 2001-2007 */
+/* Copyright Gerhard Rieger 2001-2008 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* translate socket addresses into human readable form */
@@ -106,6 +106,7 @@ void socket_in6_init(struct sockaddr_in6 *sa) {
    length of the specific socket address, or 0 on error. */
 socklen_t socket_init(int af, union sockaddr_union *sa) {
    switch (af) {
+   case AF_UNSPEC: memset(sa, 0, sizeof(*sa)); return sizeof(*sa);
 #if WITH_UNIX
    case AF_UNIX:   socket_un_init(&sa->un);   return sizeof(sa->un);
 #endif
@@ -115,7 +116,7 @@ socklen_t socket_init(int af, union sockaddr_union *sa) {
 #if WITH_IP6
    case AF_INET6:  socket_in6_init(&sa->ip6); return sizeof(sa->ip6);
 #endif
-   default: Error1("socket_init(): unknown address family %d", af);
+   default: Info1("socket_init(): unknown address family %d", af);
       memset(sa, 0, sizeof(union sockaddr_union));
       sa->soa.sa_family = af;
       return 0;
@@ -129,64 +130,65 @@ socklen_t socket_init(int af, union sockaddr_union *sa) {
 
 #if _WITH_SOCKET
 char *sockaddr_info(const struct sockaddr *sa, socklen_t salen, char *buff, size_t blen) {
-   char ubuff[5*UNIX_PATH_MAX+3];
+   union sockaddr_union *sau = (union sockaddr_union *)sa;
    char *lbuff = buff;
    char *cp = lbuff;
    int n;
 
-   if ((n = snprintf(cp, blen, "AF=%d ", sa->sa_family)) < 0) {
+#if HAVE_STRUCT_SOCKADDR_SALEN
+   if ((n = snprintf(cp, blen, "LEN=%d ", sau->soa.sa_len)) < 0) {
+      Warn1("sockaddr_info(): buffer too short ("F_Zu")", blen);
+      *buff = '\0';
+      return buff;
+   }
+   cp += n,  blen -= n;
+#endif
+   if ((n = snprintf(cp, blen, "AF=%d ", sau->soa.sa_family)) < 0) {
       Warn1("sockaddr_info(): buffer too short ("F_Zu")", blen);
       *buff = '\0';
       return buff;
    }
    cp += n,  blen -= n;
 
-   switch (sa->sa_family) {
+   switch (sau->soa.sa_family) {
 #if WITH_UNIX
    case 0:
-   case AF_UNIX:
-#if WITH_ABSTRACT_UNIXSOCKET
-      if (salen > XIOUNIXSOCKOVERHEAD &&
-	  sa->sa_data[0] == '\0') {
-	 char *nextc;
-//	 nextc =
-//	    sanitize_string((char *)&sa->sa_data+1, salen-XIOUNIXSOCKOVERHEAD-1,
-//			    ubuff, XIOSAN_DEFAULT_BACKSLASH_OCT_3);
-	 nextc =
-	    sanitize_string((char *)&sa->sa_data, salen-XIOUNIXSOCKOVERHEAD,
-			    ubuff, XIOSAN_DEFAULT_BACKSLASH_OCT_3);
-	 *nextc = '\0';
-//	 snprintf(cp, blen, "\"\\0%s\"", ubuff);
-	 snprintf(cp, blen, "\"%s\"", ubuff);
-      } else
-#endif /* WITH_ABSTRACT_UNIXSOCKET */
-      {
-	 char *nextc;
-	 nextc =
-	    sanitize_string((char *)&sa->sa_data,
-			    MIN(UNIX_PATH_MAX, strlen((char *)&sa->sa_data)),
-			    ubuff, XIOSAN_DEFAULT_BACKSLASH_OCT_3);
-	 *nextc = '\0';
-	 snprintf(cp, blen, "\"%s\"", ubuff);
-      }
+   case AF_UNIX: sockaddr_unix_info(&sau->un, salen, cp+1, blen-1);
+      cp[0] = '"';
+      *strchr(cp+1, '\0') = '"';
       break;
 #endif
 #if WITH_IP4
-   case AF_INET: sockaddr_inet4_info((struct sockaddr_in *)sa, cp, blen);
+   case AF_INET: sockaddr_inet4_info(&sau->ip4, cp, blen);
       break;
 #endif
 #if WITH_IP6
-   case AF_INET6: sockaddr_inet6_info((struct sockaddr_in6 *)sa, cp, blen);
+   case AF_INET6: sockaddr_inet6_info(&sau->ip6, cp, blen);
       break;
 #endif
    default:
+      if ((n = snprintf(cp, blen, "AF=%d ", sa->sa_family)) < 0) {
+	 Warn1("sockaddr_info(): buffer too short ("F_Zu")", blen);
+	 *buff = '\0';
+	 return buff;
+      }
+      cp += n,  blen -= n;
       if ((snprintf(cp, blen,
 		    "0x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
-		    sa->sa_data[0], sa->sa_data[1], sa->sa_data[2],
-		    sa->sa_data[3], sa->sa_data[4], sa->sa_data[5],
-		    sa->sa_data[6], sa->sa_data[7], sa->sa_data[8],
-		    sa->sa_data[9], sa->sa_data[10], sa->sa_data[11],
-		    sa->sa_data[12], sa->sa_data[13])) < 0) {
+		    ((unsigned char *)sau->soa.sa_data)[0],
+		    ((unsigned char *)sau->soa.sa_data)[1],
+		    ((unsigned char *)sau->soa.sa_data)[2],
+		    ((unsigned char *)sau->soa.sa_data)[3],
+		    ((unsigned char *)sau->soa.sa_data)[4],
+		    ((unsigned char *)sau->soa.sa_data)[5],
+		    ((unsigned char *)sau->soa.sa_data)[6],
+		    ((unsigned char *)sau->soa.sa_data)[7],
+		    ((unsigned char *)sau->soa.sa_data)[8],
+		    ((unsigned char *)sau->soa.sa_data)[9],
+		    ((unsigned char *)sau->soa.sa_data)[10],
+		    ((unsigned char *)sau->soa.sa_data)[11],
+		    ((unsigned char *)sau->soa.sa_data)[12],
+		    ((unsigned char *)sau->soa.sa_data)[13])) < 0) {
 	 Warn("sockaddr_info(): buffer too short");
 	 *buff = '\0';
 	 return buff;
@@ -199,10 +201,26 @@ char *sockaddr_info(const struct sockaddr *sa, socklen_t salen, char *buff, size
 
 #if WITH_UNIX
 char *sockaddr_unix_info(const struct sockaddr_un *sa, socklen_t salen, char *buff, size_t blen) {
-   blen = Min(blen, sizeof(sa->sun_path));
-   strncpy(buff, sa->sun_path, blen);
-   if (strlen(buff) >= blen) {
-      buff[blen-1] = '\0';
+   char ubuff[5*UNIX_PATH_MAX+3];
+   char *nextc;
+
+#if WITH_ABSTRACT_UNIXSOCKET
+   if (salen > XIOUNIXSOCKOVERHEAD &&
+       sa->sun_path[0] == '\0') {
+      nextc =
+	 sanitize_string(sa->sun_path, salen-XIOUNIXSOCKOVERHEAD,
+			 ubuff, XIOSAN_DEFAULT_BACKSLASH_OCT_3);
+      *nextc = '\0';
+      strncpy(buff, ubuff, blen);
+   } else
+#endif /* WITH_ABSTRACT_UNIXSOCKET */
+   {
+      nextc =
+	 sanitize_string(sa->sun_path,
+			 MIN(UNIX_PATH_MAX, strlen(sa->sun_path)),
+			 ubuff, XIOSAN_DEFAULT_BACKSLASH_OCT_3);
+      *nextc = '\0';
+      strncpy(buff, ubuff, blen);
    }
    return buff;
 }
@@ -253,6 +271,7 @@ const char *inet_ntop(int pf, const void *binaddr,
 	 return NULL;	/* errno is valid */
       }
       break;
+#if WITH_IP6
    case PF_INET6:
       if ((retlen =
 	   snprintf(addrtext, textlen, "%x:%x:%x:%x:%x:%x:%x:%x",
@@ -269,6 +288,7 @@ const char *inet_ntop(int pf, const void *binaddr,
 	 return NULL;	/* errno is valid */
       }
       break;
+#endif /* WITH_IP6 */
    default:
       errno = EAFNOSUPPORT;
       return NULL;
@@ -280,7 +300,7 @@ const char *inet_ntop(int pf, const void *binaddr,
 
 #if WITH_IP6
 /* convert the IP6 socket address to human readable form. buff should be at
-   least 50 chars long */
+   least 50 chars long. output includes the port number */
 char *sockaddr_inet6_info(const struct sockaddr_in6 *sa, char *buff, size_t blen) {
    if (snprintf(buff, blen, "[%04x:%04x:%04x:%04x:%04x:%04x:%04x:%04x]:%hu",
 #if HAVE_IP6_SOCKADDR==0
@@ -400,10 +420,68 @@ const char *hstrerror(int err) {
    return h_messages[err];
 }
 #endif /* !HAVE_HSTRERROR */
+
+
+/* this function behaves like poll(). It tries to do so even when the poll()
+   system call is not available. */
+/* note: glibc 5.4 does not know nfds_t */
+int xiopoll(struct pollfd fds[], unsigned long nfds, struct timeval *timeout) {
+   int i, n = 0;
+   int result = 0;
+
+   while (true) { /* should be if (), but we want to break */
+      fd_set readfds;
+      fd_set writefds;
+      fd_set exceptfds;
+
+      FD_ZERO(&readfds);  FD_ZERO(&writefds);  FD_ZERO(&exceptfds);
+      for (i = 0; i < nfds; ++i) {
+	 fds[i].revents = 0;
+	 if (fds[i].fd < 0)  { continue; }
+	 if (fds[i].fd > FD_SETSIZE)  { break; /* use poll */ }
+	 if (fds[i].events & POLLIN)  {
+	    FD_SET(fds[i].fd, &readfds);  n = MAX(n, fds[i].fd); }
+	 if (fds[i].events & POLLOUT) {
+	    FD_SET(fds[i].fd, &writefds); n = MAX(n, fds[i].fd); }
+      }
+      if (i < nfds)  { break; /* use poll */ }
+
+      result = Select(n+1, &readfds, &writefds, &exceptfds, timeout);
+      if (result < 0)  { return result; }
+      for (i = 0; i < nfds; ++i) {
+	 if (fds[i].fd < 0)  { continue; }
+	 if ((fds[i].events & POLLIN)  && FD_ISSET(fds[i].fd, &readfds))  {
+	    fds[i].revents |= POLLIN;  ++result;
+	 }
+	 if ((fds[i].events & POLLOUT) && FD_ISSET(fds[i].fd, &writefds)) {
+	    fds[i].revents |= POLLOUT; ++result;
+	 }
+      }
+      return result;
+   }
+#if HAVE_POLL
+   {
+      int ms = 0;
+      if (timeout == NULL) {
+	 ms = -1;
+      } else {
+	 ms = 1000*timeout->tv_sec + timeout->tv_usec/1000;
+      }
+      /*! timeout */
+      return Poll(fds, nfds, ms);
+#else /* HAVE_POLL */
+   } else {
+      Error("poll() not available");
+      return -1;
+#endif /* !HAVE_POLL */
+   }
+}
    
 
 #if WITH_TCP || WITH_UDP
-/* returns port in network byte order */
+/* returns port in network byte order;
+   ipproto==IPPROTO_UDP resolves as UDP service, every other value resolves as
+   TCP */
 int parseport(const char *portname, int ipproto) {
    struct servent *se;
    char *extra;
@@ -418,7 +496,7 @@ int parseport(const char *portname, int ipproto) {
       return result;
    }
 
-   if ((se = getservbyname(portname, ipproto==IPPROTO_TCP?"tcp":"udp")) == NULL) {
+   if ((se = getservbyname(portname, ipproto==IPPROTO_UDP?"udp":"tcp")) == NULL) {
       Error2("cannot resolve service \"%s/%d\"", portname, ipproto);
       return 0;
    }
@@ -427,10 +505,15 @@ int parseport(const char *portname, int ipproto) {
 }
 #endif /* WITH_TCP || WITH_UDP */
 
-#if WITH_IP4 || WITH_IP6
+
+#if WITH_IP4 || WITH_IP6 || WITH_INTERFACE
 /* check the systems interfaces for ifname and return its index
-   or -1 if no interface with this name was found */
-int ifindexbyname(const char *ifname) {
+   or -1 if no interface with this name was found
+   The system calls require an arbitrary socket; the calling program may
+   provide one in anysock to avoid creation of a dummy socket. anysock must be
+   <0 if it does not specify a socket fd.
+ */
+int ifindexbyname(const char *ifname, int anysock) {
    /* Linux: man 7 netdevice */
    /* FreeBSD: man 4 networking */
    /* Solaris: man 7 if_tcp */
@@ -438,29 +521,35 @@ int ifindexbyname(const char *ifname) {
 #if defined(HAVE_STRUCT_IFREQ) && defined(SIOCGIFCONF) && defined(SIOCGIFINDEX)
    /* currently we support Linux, FreeBSD; not Solaris */
 
-#define IFBUFSIZ 1024
+#define IFBUFSIZ 32*sizeof(struct ifreq) /*1024*/
    int s;
    struct ifreq ifr;
 
    if (ifname[0] == '\0') {
       return -1;
    }
-   if ((s = Socket(PF_INET, SOCK_DGRAM, IPPROTO_IP)) < 0) {
+   if (anysock >= 0) {
+      s = anysock;
+   } else  if ((s = Socket(PF_INET, SOCK_DGRAM, IPPROTO_IP)) < 0) {
       Error1("socket(PF_INET, SOCK_DGRAM, IPPROTO_IP): %s", strerror(errno));
       return -1;
    }
 
    strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
    if (Ioctl(s, SIOCGIFINDEX, &ifr) < 0) {
-      Close(s);
-      Info3("ioctl(%d, SIOCGIFINDEX, {%s}): %s",
+      Info3("ioctl(%d, SIOCGIFINDEX, {\"%s\"}): %s",
 	     s, ifr.ifr_name, strerror(errno));
+      Close(s);
       return -1;
    }
    Close(s);
 #if HAVE_STRUCT_IFREQ_IFR_INDEX
+   Info3("ioctl(%d, SIOCGIFINDEX, {\"%s\"}) -> { %d }",
+         s, ifname, ifr.ifr_index);
    return ifr.ifr_index;
 #elif HAVE_STRUCT_IFREQ_IFR_IFINDEX
+   Info3("ioctl(%d, SIOCGIFINDEX, {\"%s\"}) -> { %d }",
+         s, ifname, ifr.ifr_ifindex);
    return ifr.ifr_ifindex;
 #endif /* HAVE_STRUCT_IFREQ_IFR_IFINDEX */
 
@@ -468,12 +557,15 @@ int ifindexbyname(const char *ifname) {
    return -1;
 #endif /* !defined(HAVE_ STRUCT_IFREQ) && defined(SIOCGIFCONF) && defined(SIOCGIFINDEX) */
 }
+#endif /* WITH_IP4 || WITH_IP6 || WITH_INTERFACE */
 
 
-/* like ifindexbyname(), but allows an index number as input.
+#if WITH_IP4 || WITH_IP6 || WITH_INTERFACE
+/* like ifindexbyname(), but also allows the index number as input - in this
+   case it does not lookup the index.
    writes the resulting index to *ifindex and returns 0,
    or returns -1 on error */
-int ifindex(const char *ifname, unsigned int *ifindex) {
+int ifindex(const char *ifname, unsigned int *ifindex, int anysock) {
    char *endptr;
    long int val;
 
@@ -486,10 +578,91 @@ int ifindex(const char *ifname, unsigned int *ifindex) {
       return 0;
    }
 
-   if ((val = ifindexbyname(ifname)) < 0) {
+   if ((val = ifindexbyname(ifname, anysock)) < 0) {
       return -1;
    }
    *ifindex = val;
    return 0;
 }
-#endif /* WITH_IP4 || WITH_IP6 */
+#endif /* WITH_IP4 || WITH_IP6 || WITH_INTERFACE */
+
+
+/* constructs an environment variable whose name is built from socats uppercase
+   program name, and underscore and varname; if a variable of this name already
+   exists a non zero value of overwrite lets the old value be overwritten.
+   returns 0 on success or <0 if an error occurred. */
+int xiosetenv(const char *varname, const char *value, int overwrite) {
+#  define XIO_ENVNAMELEN 256
+   const char *progname;
+   char envname[XIO_ENVNAMELEN];
+   size_t i, l;
+
+   progname = diag_get_string('p');
+   strncpy(envname, progname, XIO_ENVNAMELEN-1);
+   l = strlen(progname);
+   strncpy(envname+l, "_", XIO_ENVNAMELEN-1-l);
+   for (i = 0; i < l; ++i)  envname[i] = toupper(envname[i]);
+   strncpy(envname+l+1, varname, XIO_ENVNAMELEN-1-l);
+   if (Setenv(envname, value, overwrite) < 0) {
+      Warn3("setenv(\"%s\", \"%s\", 1): %s",
+	    envname, value, strerror(errno));
+#if HAVE_UNSETENV
+      Unsetenv(envname);      /* dont want to have a wrong value */
+#endif
+      return -1;
+   }
+   return 0;
+#  undef XIO_ENVNAMELEN
+}
+
+int xiosetenv2(const char *varname, const char *varname2, const char *value,
+	       int overwrite) {
+#  define XIO_ENVNAMELEN 256
+   const char *progname;
+   char envname[XIO_ENVNAMELEN];
+   size_t i, l;
+
+   progname = diag_get_string('p');
+   strncpy(envname, progname, XIO_ENVNAMELEN-1);
+   l = strlen(progname);
+   strncpy(envname+l, "_", XIO_ENVNAMELEN-1-l);
+   l += 1;
+   strncpy(envname+l, varname, XIO_ENVNAMELEN-1-l);
+   l += strlen(varname);
+   strncpy(envname+l, "_", XIO_ENVNAMELEN-1-l);
+   l += 1;
+   strncpy(envname+l, varname2, XIO_ENVNAMELEN-1-l);
+   l += strlen(varname2);
+   for (i = 0; i < l; ++i)  envname[i] = toupper(envname[i]);
+   if (Setenv(envname, value, overwrite) < 0) {
+      Warn3("setenv(\"%s\", \"%s\", 1): %s",
+	    envname, value, strerror(errno));
+#if HAVE_UNSETENV
+      Unsetenv(envname);      /* dont want to have a wrong value */
+#endif
+      return -1;
+   }
+   return 0;
+#  undef XIO_ENVNAMELEN
+}
+
+
+/* like xiosetenv(), but uses an unsigned long value */
+int xiosetenvulong(const char *varname, unsigned long value, int overwrite) {
+#  define XIO_LONGLEN 21	/* should suffice for 64bit longs with \0 */
+   char envbuff[XIO_LONGLEN];
+
+   snprintf(envbuff, XIO_LONGLEN, "%lu", value);
+   return xiosetenv(varname, envbuff, overwrite);
+#  undef XIO_LONGLEN
+}
+
+/* like xiosetenv(), but uses an unsigned short value */
+int xiosetenvushort(const char *varname, unsigned short value, int overwrite) {
+#  define XIO_SHORTLEN 11      /* should suffice for 32bit shorts with \0 */
+   char envbuff[XIO_SHORTLEN];
+
+   snprintf(envbuff, XIO_SHORTLEN, "%hu", value);
+   return xiosetenv(varname, envbuff, overwrite);
+#  undef XIO_SHORTLEN
+}
