@@ -1,5 +1,5 @@
-/* $Id: xio-udp.c,v 1.36 2007/02/08 18:27:00 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2007 */
+/* source: xio-udp.c */
+/* Copyright Gerhard Rieger 2001-2008 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for handling UDP addresses */
@@ -84,7 +84,7 @@ int xioopen_ipdgram_listen(int argc, const char *argv[], struct opt *opts,
    union sockaddr_union themunion;
    union sockaddr_union *them = &themunion;
    int socktype = SOCK_DGRAM;
-   fd_set in, out, expt;
+   struct pollfd readfd;
    bool dofork = false;
    pid_t pid;
    char *rangename;
@@ -114,7 +114,6 @@ int xioopen_ipdgram_listen(int argc, const char *argv[], struct opt *opts,
    applyopts(-1, opts, PH_INIT);
 
    uslen = socket_init(pf, &us);
-   retropt_int(opts, OPT_SO_TYPE, &socktype);
    retropt_bind(opts, pf, socktype, IPPROTO_UDP,
 		(struct sockaddr *)&us, &uslen, 1,
 		fd->stream.para.socket.ip.res_opts[1],
@@ -145,7 +144,7 @@ int xioopen_ipdgram_listen(int argc, const char *argv[], struct opt *opts,
 
 #if WITH_IP4 /*|| WITH_IP6*/
    if (retropt_string(opts, OPT_RANGE, &rangename) >= 0) {
-      if (parserange(rangename, pf, &fd->stream.para.socket.range) < 0) {
+      if (xioparserange(rangename, pf, &fd->stream.para.socket.range) < 0) {
 	 free(rangename);
 	 return STAT_NORETRY;
       }
@@ -164,6 +163,10 @@ int xioopen_ipdgram_listen(int argc, const char *argv[], struct opt *opts,
    }
    retropt_bool(opts, OPT_LOWPORT, &fd->stream.para.socket.ip.lowport);
 
+   if (dofork) {
+      xiosetchilddied();	/* set SIGCHLD handler */
+   }
+
    while (true) {	/* we loop with fork or prohibited packets */
       /* now wait for some packet on this datagram socket, get its sender
 	 address, connect there, and return */
@@ -172,8 +175,7 @@ int xioopen_ipdgram_listen(int argc, const char *argv[], struct opt *opts,
       union sockaddr_union _sockname;
       union sockaddr_union *la = &_sockname;	/* local address */
 
-      if ((fd->stream.fd = Socket(pf, socktype, ipproto)) < 0) {
-	 Error4("socket(%d, %d, %d): %s", pf, socktype, ipproto, strerror(errno));
+      if ((fd->stream.fd = xiosocket(opts, pf, socktype, ipproto, E_ERROR)) < 0) {
 	 return STAT_RETRYLATER;
       }
       applyopts(fd->stream.fd, opts, PH_PASTSOCKET);
@@ -201,9 +203,9 @@ int xioopen_ipdgram_listen(int argc, const char *argv[], struct opt *opts,
 
       Notice1("listening on UDP %s",
 	      sockaddr_info(&us.soa, uslen, infobuff, sizeof(infobuff)));
-      FD_ZERO(&in); FD_ZERO(&out); FD_ZERO(&expt);
-      FD_SET(fd->stream.fd, &in);
-      while (Select(fd->stream.fd+1, &in, &out, &expt, NULL) < 0) {
+      readfd.fd = fd->stream.fd;
+      readfd.events = POLLIN|POLLERR;
+      while (xiopoll(&readfd, 1, NULL) < 0) {
 	 if (errno != EINTR)  break;
       }
 
@@ -226,33 +228,29 @@ int xioopen_ipdgram_listen(int argc, const char *argv[], struct opt *opts,
       if (xiocheckpeer(&fd->stream, them, la) < 0) {
 	 /* drop packet */
 	 char buff[512];
-	 Recv(fd->stream.fd, buff, sizeof(buff), 0);
+	 Recv(fd->stream.fd, buff, sizeof(buff), 0);	/* drop packet */
+	 Close(fd->stream.fd);
 	 continue;
       }
       Info1("permitting UDP connection from %s",
 	    sockaddr_info(&them->soa, themlen, infobuff, sizeof(infobuff)));
 
       if (dofork) {
-	 pid = Fork();
+	 pid = xio_fork(false, E_ERROR);
 	 if (pid < 0) {
-	    Error1("fork(): %s", strerror(errno));
 	    return STAT_RETRYLATER;
 	 }
-	 if (pid == 0) {	/* child */
 
-	    /* drop parents locks, reset FIPS... */
-	    if (xio_forked_inchild() != 0) {
-	       Exit(1);
-	    }
+	 if (pid == 0) {	/* child */
 	    break;
 	 }
-	 /* server: continue loop with select */
+
+	 /* server: continue loop with socket()+recvfrom() */
 	 /* when we dont close this we get awkward behaviour on Linux 2.4:
 	    recvfrom gives 0 bytes with invalid socket address */
 	 if (Close(fd->stream.fd) < 0) {
 	    Info2("close(%d): %s", fd->stream.fd, strerror(errno));
 	 }
-	 Notice1("forked off child process "F_pid, pid);
 	 Sleep(1);	/*! give child a chance to consume the old packet */
 
 	 continue;
@@ -269,6 +267,14 @@ int xioopen_ipdgram_listen(int argc, const char *argv[], struct opt *opts,
       return STAT_RETRYLATER;
    }
 
+   /* set the env vars describing the local and remote sockets */
+   if (Getsockname(fd->stream.fd, &us.soa, &uslen) < 0) {
+      Warn4("getsockname(%d, %p, {%d}): %s",
+	    fd->stream.fd, &us.soa, uslen, strerror(errno));
+   }
+   xiosetsockaddrenv("SOCK", &us,  uslen,   IPPROTO_UDP);
+   xiosetsockaddrenv("PEER", them, themlen, IPPROTO_UDP);
+
    fd->stream.howtoend = END_SHUTDOWN;
    applyopts_fchown(fd->stream.fd, opts);
    applyopts(fd->stream.fd, opts, PH_LATE);
@@ -291,6 +297,8 @@ int xioopen_udp_sendto(int argc, const char *argv[], struct opt *opts,
 	     argv[0], argc-1);
       return STAT_NORETRY;
    }
+
+   retropt_socket_pf(opts, &pf);
    if ((result = _xioopen_udp_sendto(argv[1], argv[2], opts, xioflags, xxfd,
 				     groups, pf, socktype, ipproto))
        != STAT_OK) {
@@ -300,6 +308,12 @@ int xioopen_udp_sendto(int argc, const char *argv[], struct opt *opts,
    return STAT_OK;
 }
 
+/*
+   applies and consumes the following option:
+   PH_INIT, PH_PASTSOCKET, PH_FD, PH_PREBIND, PH_BIND, PH_PASTBIND, PH_CONNECTED, PH_LATE
+   OFUNC_OFFSET
+   OPT_BIND, OPT_SOURCEPORT, OPT_LOWPORT, OPT_SO_TYPE, OPT_SO_PROTOTYPE, OPT_USER, OPT_GROUP, OPT_CLOEXEC
+ */
 static
 int _xioopen_udp_sendto(const char *hostname, const char *servname,
 			struct opt *opts,
@@ -313,7 +327,6 @@ int _xioopen_udp_sendto(const char *hostname, const char *servname,
    int result;
 
    xfd->howtoend = END_SHUTDOWN;
-   retropt_int(opts, OPT_SO_TYPE, &socktype);
 
    /* ...res_opts[] */
    if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
@@ -400,6 +413,7 @@ int xioopen_udp_datagram(int argc, const char *argv[], struct opt *opts,
       return STAT_RETRYLATER;
    }
 
+   retropt_socket_pf(opts, &pf);
    result =
       _xioopen_udp_sendto(hostname, argv[2], opts, xioflags, xxfd, groups,
 			 pf, socktype, ipproto);
@@ -419,7 +433,7 @@ int xioopen_udp_datagram(int argc, const char *argv[], struct opt *opts,
    /* which reply packets will be accepted - determine by range option */
    if (retropt_string(opts, OPT_RANGE, &rangename)
        >= 0) {
-      if (parserange(rangename, pf, &xfd->para.socket.range) < 0) {
+      if (xioparserange(rangename, pf, &xfd->para.socket.range) < 0) {
 	 free(rangename);
 	 return STAT_NORETRY;
       }
@@ -577,7 +591,7 @@ int xioopen_udp_recv(int argc, const char *argv[], struct opt *opts,
 
 #if WITH_IP4 /*|| WITH_IP6*/
    if (retropt_string(opts, OPT_RANGE, &rangename) >= 0) {
-      if (parserange(rangename, pf, &xfd->stream.para.socket.range) < 0) {
+      if (xioparserange(rangename, pf, &xfd->stream.para.socket.range) < 0) {
 	 return STAT_NORETRY;
       }
       xfd->stream.para.socket.dorange = true;
