@@ -10,6 +10,7 @@
 #define ODESC_ERROR ODESC_DONE	/* maybe later */
 
 #define XIO_OFFSETOF(x) ((size_t)&((xiosingle_t *)0)->x)
+#define XIO_SIZEOF(x) (sizeof(((struct single *)0)->x))
 
 /* atomic structure for use in the option search table; keep compatible with
    struct wordent! */
@@ -20,7 +21,7 @@ struct optname {
 
 /* keep consistent with xiohelp.c:optiontypenames[] ! */
 enum e_types {
-   TYPE_CONST,		/* keyword means a fix value */
+   TYPE_CONST,		/* keyword means a fix value - implies int type */
    TYPE_BIN,		/* raw binary data, length determined by data */
    TYPE_BOOL,		/* value is 0 or 1 (no-value is interpreted as 1) */
    TYPE_BYTE,		/* unsigned char */
@@ -935,4 +936,52 @@ extern int _groupbits(mode_t mode);
 extern int dropopts(struct opt *opts, unsigned int phase);
 extern int dropopts2(struct opt *opts, unsigned int from, unsigned int to);
 
+#if HAVE_BASIC_UID_T==1
+#  define retropt_uid(o,c,r) retropt_short(o,c,r)
+#elif HAVE_BASIC_UID_T==2
+#  define retropt_uid(o,c,r) retropt_ushort(o,c,r)
+#elif HAVE_BASIC_UID_T==3
+#  define retropt_uid(o,c,r) retropt_int(o,c,r)
+#elif HAVE_BASIC_UID_T==4
+#  define retropt_uid(o,c,r) retropt_uint(o,c,r)
+#elif HAVE_BASIC_UID_T==5
+#  define retropt_uid(o,c,r) retropt_long(o,c,r)
+#elif HAVE_BASIC_UID_T==6
+#  define retropt_uid(o,c,r) retropt_ulong(o,c,r)
+#else
+#  error "HAVE_BASIC_UID_T is out of range: " HAVE_BASIC_UID_T
+#endif
+
+#if HAVE_BASIC_GID_T==1
+#  define retropt_gid(o,c,r) retropt_short(o,c,r)
+#elif HAVE_BASIC_GID_T==2
+#  define retropt_gid(o,c,r) retropt_ushort(o,c,r)
+#elif HAVE_BASIC_GID_T==3
+#  define retropt_gid(o,c,r) retropt_int(o,c,r)
+#elif HAVE_BASIC_GID_T==4
+#  define retropt_gid(o,c,r) retropt_uint(o,c,r)
+#elif HAVE_BASIC_GID_T==5
+#  define retropt_gid(o,c,r) retropt_long(o,c,r)
+#elif HAVE_BASIC_GID_T==6
+#  define retropt_gid(o,c,r) retropt_ulong(o,c,r)
+#else
+#  error "HAVE_BASIC_GID_T is out of range: " HAVE_BASIC_GID_T
+#endif
+
+#if HAVE_BASIC_MODE_T==1
+#  define retropt_mode(o,c,r) retropt_short(o,c,r)
+#elif HAVE_BASIC_MODE_T==2
+#  define retropt_mode(o,c,r) retropt_ushort(o,c,r)
+#elif HAVE_BASIC_MODE_T==3
+#  define retropt_mode(o,c,r) retropt_int(o,c,r)
+#elif HAVE_BASIC_MODE_T==4
+#  define retropt_mode(o,c,r) retropt_uint(o,c,r)
+#elif HAVE_BASIC_MODE_T==5
+#  define retropt_mode(o,c,r) retropt_long(o,c,r)
+#elif HAVE_BASIC_MODE_T==6
+#  define retropt_mode(o,c,r) retropt_ulong(o,c,r)
+#else
+#  error "HAVE_BASIC_MODE_T is out of range: " HAVE_BASIC_MODE_T
+#endif
+
 #endif /* !defined(__xioopts_h_included) */
