@@ -1,5 +1,5 @@
 /* source: xioshutdown.c */
-/* Copyright Gerhard Rieger 2001-2007 */
+/* Copyright Gerhard Rieger 2001-2009 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this is the source of the extended shutdown function */
@@ -31,7 +31,36 @@ int xioshutdown(xiofile_t *sock, int how) {
       if ((how+1)&2) {
 	 result |= xioshutdown((xiofile_t *)sock->dual.stream[1], 1);
       }
+      return result;
+   }
 
+   switch (sock->stream.howtoshut) {
+      char writenull;
+   case XIOSHUT_NONE:
+      return 0;
+   case XIOSHUT_CLOSE:
+      if (Close(sock->stream.fd) < 0) {
+	 Info2("close(%d): %s",
+	       sock->stream.fd, strerror(errno));
+      }
+      return 0;
+   case XIOSHUT_DOWN:
+      if ((result = Shutdown(sock->stream.fd, how)) < 0) {
+	 Info3("shutdown(%d, %d): %s",
+	       sock->stream.fd, how, strerror(errno));
+      }
+      return 0;
+#if _WITH_SOCKET
+   case XIOSHUT_NULL:
+      /* send an empty packet; only useful on datagram sockets? */
+      xiowrite(sock, &writenull, 0);
+      return 0;
+#endif /* _WITH_SOCKET */
+   default: ;
+   }
+
+   if (false) {
+      ;
 #if WITH_OPENSSL
    } else if ((sock->stream.dtype & XIODATA_MASK) == XIODATA_OPENSSL) {
       sycSSL_shutdown (sock->stream.para.openssl.ssl);
@@ -65,7 +94,7 @@ int xioshutdown(xiofile_t *sock, int how) {
 		  sock->stream.para.exec.fdout, strerror(errno));
 	 } 
       }
-#if WITH_SOCKET
+#if _WITH_SOCKET
    } else if (sock->stream.howtoend == END_SHUTDOWN) {
       if ((result = Shutdown(sock->stream.fd, how)) < 0) {
 	 Info3("shutdown(%d, %d): %s",
@@ -107,7 +136,7 @@ int xioshutdown(xiofile_t *sock, int how) {
 	 sock->stream.eof = 2;
 	 sock->stream.fd = -1;
       }
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
 #if 0
    } else {
       Error1("xioshutdown(): bad data type specification %d", sock->stream.dtype);
